package Communication;


import java.awt.image.PixelGrabber;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Vector;
import UI_Frames.FrameSelection;
import Element.ElementImage;
import Element.Order;
import Element.Product;
import Element.ProductSt;
import Element.Recommendation;

import Element.UserInfo;
import Element.UserInformation;
import Element.UserPreference;

public class SocketThread extends Thread {
	@SuppressWarnings("unused")
	private FrameSelection selectionFrame;
	private Socket socket;
	private ObjectInputStream in;
	private ObjectOutputStream out;

	private boolean stop = true;
	

	public SocketThread(FrameSelection selectionFrame, String ip) {
		this.selectionFrame = selectionFrame;
		//String addr = new String("127.0.0.1");
		try {
			socket = new Socket(ip, 9000);
			out = new ObjectOutputStream(socket.getOutputStream());
			in = new ObjectInputStream(socket.getInputStream());
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void sendString(String str) {
		try {
			this.out.writeObject(str);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void sendUserPrf(UserPreference UPf){
		UserPreference u = new UserPreference();
		
		u.setStyle(UPf.getStyle());
		u.setActivityValue(UPf.getActivityValue());
		u.setBrandValue(UPf.getBrandValue());
		u.setColorfulValue(UPf.getColorfulValue());
		u.setPriceValue(UPf.getPriceValue());
		u.setSplendorValue(UPf.getSplendorValue());
		u.setSeanson(UPf.getSeason());
		u.setWeather(UPf.getWeather());
		try {
			out.writeObject(u);
		} catch (IOException e) {
			e.printStackTrace();
		}
	
	}
	
	public void sendOrder(Order order){
		try {
			//System.out.println("socket false left:"+order.getIsNext());
			out.writeObject(order);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void sendUser(UserInfo user){
		try {
			out.writeObject(user);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public void sendRecommend(Recommendation r) {
		try {
			out.writeObject(r);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	public void close(){
		try {
			this.out.writeObject("end");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	public void requestRecent(){
		try {
			this.out.writeObject("recent");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void sendUserInformation(UserInformation user) {
		try {
			this.out.writeObject(user);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
	}
	
	public void requestPopularProduct(){
		try {
			this.out.writeObject("popular");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public void sendImage(ElementImage element) {

		int imageWidth = element.imageWidth;
		int imageHeight = element.imageHeight;
		element.imagePixel = new int[imageWidth * imageHeight]; // imagePixel
		PixelGrabber grabber = new PixelGrabber(element.image, 0, 0,
				imageWidth, imageHeight, element.imagePixel, 0, imageWidth);
		try {
			grabber.grabPixels();
			element.image = null;
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			this.out.writeObject(element);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@SuppressWarnings("unchecked")
	public void run() {
		while (stop) {
			try {
				Object o = in.readObject();
				if (o instanceof String) {
					String str = (String)o;
					if(str.startsWith("end")){
						stop = false;
					}
				}else if (o instanceof Product) {
					Product p = (Product)o;
					p.ByteToImage(this.selectionFrame.displayFrame.displayPanel);
					this.selectionFrame.displayFrame.displayPanel.addProduct(p);
					this.selectionFrame.displayFrame.displayPanel.displayProduct();
				}
				else if(o instanceof Vector){
					Vector<Product> rp= (Vector<Product>)o;
					if(rp.size() == 10){
						this.selectionFrame.panelRecent.setProductList(rp);
					}else if(rp.size() == 9){
						this.selectionFrame.panelPopular.setProductList(rp);
					}
					
				}
				else if (o instanceof ArrayList) {
					ArrayList<ProductSt> ProductList = (ArrayList<ProductSt>)o;
					
					for(int i=0;i<ProductList.size();i++){
						ProductList.get(i).getTop().ByteToImage(selectionFrame.displayFrame.displayPanel);
						ProductList.get(i).getBottom().ByteToImage(selectionFrame.displayFrame.displayPanel);
						ProductList.get(i).getAccessary().ByteToImage(selectionFrame.displayFrame.displayPanel);
					}
					
					this.selectionFrame.displayFrame.displayPanel.setProductList(ProductList);
					
				}else if(o instanceof ProductSt){
					ProductSt ps = (ProductSt)o;
					
					ps.getTop().ByteToImage(selectionFrame.displayFrame.displayPanel);
					ps.getBottom().ByteToImage(selectionFrame.displayFrame.displayPanel);
					ps.getAccessary().ByteToImage(selectionFrame.displayFrame.displayPanel);
					
					//System.out.println(" pn  :" + ps.getTop().getProductNum());
					//System.out.println(" pn  :" + ps.getBottom().getProductNum());
					//System.out.println(" pn  :" + ps.getAccessary().getProductNum());
					this.selectionFrame.displayFrame.displayPanel.setProductTexture(ps);
				}else if(o instanceof UserInformation){
					UserInformation user = (UserInformation)o;
					if(user.getFlag()==true){
						this.selectionFrame.swapCard("mainPanel");
						this.selectionFrame.setUserInformation(user);
					}
					else{
						this.selectionFrame.loginPanel.loginDenial(user);
					}
					
				}else if(o instanceof UserInfo){
					//this.selectionFrame.swapCard("mainPanel");
					UserInfo user = (UserInfo)o;
					if(user.getFlag()==true){
						this.selectionFrame.loginPanel.joinframe.availableID(user);
					}
					else{
						this.selectionFrame.loginPanel.joinframe.duplicateID(user);
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}

		}
		try {
			this.socket.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	

	
}
